from django.conf import settings
try:
    import redis
    from redis.connection import ConnectionError
except ImportError:
    pass


def check(request):
    host = settings.CACHEOPS_REDIS.get('host', '')
    port = settings.CACHEOPS_REDIS.get('port', 0)
    db = settings.CACHEOPS_REDIS.get('db', 0)
    socket_timeout = settings.CACHEOPS_REDIS.get('socket_timeout')

    try:
        redis_con = redis.StrictRedis(
            host=host, port=port, db=db, socket_timeout=socket_timeout)
        ping = redis_con.ping()
    except NameError:
        return {'error': 'cannot import redis library'}
    except ConnectionError as e:
        return {'error': str(e)}

    return {
            'ping': ping,
            'version': redis_con.info().get('redis_version')
        }



import re
import base64
import logging
from functools import wraps
from ipaddress import ip_address, ip_network

from .settings import HEARTBEAT
from django.http import HttpResponse
from django.core.exceptions import ImproperlyConfigured

logging.basicConfig(
    format='%(levelname)s - %(message)s',
    level=logging.INFO)
logger = logging.getLogger(__name__)


def auth(func):
    @wraps(func)
    def _decorator(request, *args, **kwargs):
        auth = get_auth()
        if auth.get('disable', False) is True:
            return func(request, *args, **kwargs)
        if 'authorized_ips' in auth:
            ip = get_client_ip(request)
            if is_authorized(ip, auth['authorized_ips']):
                return func(request, *args, **kwargs)
        prepare_credentials(auth)
        if request.META.get('HTTP_AUTHORIZATION'):
            authmeth, auth = request.META['HTTP_AUTHORIZATION'].split(' ')
            if authmeth.lower() == 'basic':
                auth = base64.b64decode(auth).decode('utf-8')
                username, password = auth.split(':')
                if (username == HEARTBEAT['auth']['username'] and
                        password == HEARTBEAT['auth']['password']):
                    return func(request, *args, **kwargs)

        response = HttpResponse(
            "Authentication failed", status=401)
        response['WWW-Authenticate'] = 'Basic realm="Welcome to 1337"'
        return response
    return _decorator


def get_auth():
    auth = HEARTBEAT.get('auth')
    if not auth:
        raise ImproperlyConfigured('Missing auth configuration for heartbeat')
    return auth


def prepare_credentials(auth):
    if not all([auth.get('username'), auth.get('password')]):
        raise ImproperlyConfigured(
            'Username or password missing from auth configuration '
            'for heartbeat')


def get_access_route(request):
    meta = request.META
    return (
        meta.get('HTTP_X_FORWARDED_FOR') or meta.get('REMOTE_ADDR')
    ).split(',')


def get_client_ip(request):
    access_route = get_access_route(request)

    if len(access_route) == 1:
        return access_route[0]
    expression = """
        (^(?!(?:[0-9]{1,3}\.){3}[0-9]{1,3}$).*$)|  # will match non valid ipV4
        (^127\.0\.0\.1)|  # will match 127.0.0.1
        (^10\.)|  # will match 10.0.0.0 - 10.255.255.255 IP-s
        (^172\.1[6-9]\.)|  # will match 172.16.0.0 - 172.19.255.255 IP-s
        (^172\.2[0-9]\.)|  # will match 172.20.0.0 - 172.29.255.255 IP-s
        (^172\.3[0-1]\.)|  # will match 172.30.0.0 - 172.31.255.255 IP-s
        (^192\.168\.)  # will match 192.168.0.0 - 192.168.255.255 IP-s
    """
    regex = re.compile(expression, re.X)
    for ip in access_route:
        if not ip:
            # it's possible that the first value from X_FORWARDED_FOR
            # will be null, so we need to pass that value
            continue
        if regex.search(ip):
            continue
        else:
            return ip


def is_authorized(ip, authorized_ips):
    ip = ip_address(ip)

    for item in authorized_ips:
        try:
            if ip == ip_address(item):
                return True
        except ValueError:
            try:
                if ip in ip_network(item):
                    return True
            except ValueError:
                logger.warn('The "authorized_ip" list (settings.HEARTBEAT)'
                            'contains an item that is neither an ip address '
                            'nor an ip network: {}'.format(item))



import json
from collections import OrderedDict
from importlib import import_module
from django.http import HttpResponse

from .auth import auth
from .settings import HEARTBEAT


def index(request):
    return HttpResponse(content='all good in the hood')


@auth
def details(request):
    response = {}
    for checker in HEARTBEAT['checkers']:
        checker_module = import_module(checker)
        checker_name = checker_module.__name__.split('.')[-1]
        data = checker_module.check(request)
        response.update({checker_name: data})

    data = OrderedDict(sorted(response.items()))

    return HttpResponse(json.dumps(data), content_type="application/json")



from django.conf import settings
from collections import OrderedDict

try:
    from django.core.cache import caches
except ImportError:
    from django.core.cache import get_cache as caches


def check(request):
    all_stats = []
    for alias in settings.CACHES:
        server_stats = []
        if is_memcached_profile(alias):
            cache_backend = get_cache(alias)
            for server, stats in cache_backend._cache.get_stats():
                stats = debyteify(stats)
                result = OrderedDict()
                result['name'] = debyteify(server)
                result['summary'] = get_summary(stats)
                result['details'] = stats
                server_stats.append(result)
            all_stats.append(dict(alias=alias, locations=server_stats))
    return all_stats


def get_summary(stats):
    return {
        'load': get_width_ratio(stats['bytes'], stats['limit_maxbytes']),
        'miss_ratio': get_width_ratio(stats['get_misses'], stats['cmd_get'])}


def get_width_ratio(value, max_value, max_width=100):
    try:
        value = float(value)
        max_value = float(max_value)
        ratio = (value / max_value) * max_width
    except ZeroDivisionError:
        return 0
    except (ValueError, TypeError, OverflowError):
        return ''
    return ratio


def debyteify(input):
    if isinstance(input, dict):
        return {debyteify(key): debyteify(value)
                for key, value in input.items()}
    elif isinstance(input, list):
        return [debyteify(element) for element in input]
    elif isinstance(input, bytes):
        return input.decode('utf-8')
    else:
        return input


def get_cache(cache_name):
    if hasattr(caches, '__call__'):
        return caches(cache_name)
    return caches[cache_name]


def is_memcached_profile(cache_profile):
    backends = ['django.core.cache.backends.memcached.MemcachedCache',
                'django.core.cache.backends.memcached.PyLibMCCache']
    return any(
        [settings.CACHES[cache_profile]['BACKEND'] == b for b in backends])



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import sys
from glob import glob
from os.path import basename
from os.path import splitext

from setuptools import find_packages
from setuptools import setup

requires = [
    'Django>=1.6',
    'psutil==4.0.0',
]

if sys.version_info < (3, 3, 0):
    requires.append('py2-ipaddress>=3.4.0')

setup(
    name='django-heartbeat',
    version='2.0.1',
    description="Your project's heartbeat/healthcheck and dependency status",
    long_description=('A simple reusable app that checks and lists '
                      'various information about your project and its '
                      'dependencies'),
    keywords="django heartbeat health check dependency services status",
    author='PBS Core Services Test Engineers',
    author_email='andrei.pradan@3pillarglobal.com',
    url='https://github.com/pbs/django-heartbeat/',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    install_requires=requires
)
